@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.retail_v2.services.catalog_service import pagers
 from google.cloud.retail_v2.types import catalog
 from google.cloud.retail_v2.types import catalog as gcr_catalog
@@ -107,6 +112,42 @@ class CatalogServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return CatalogServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> CatalogServiceTransport:
         """Returns the transport used by the client instance.
@@ -169,18 +210,39 @@ class CatalogServiceAsyncClient:
 
     async def list_catalogs(
         self,
-        request: catalog_service.ListCatalogsRequest = None,
+        request: Union[catalog_service.ListCatalogsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListCatalogsAsyncPager:
         r"""Lists all the [Catalog][google.cloud.retail.v2.Catalog]s
         associated with the project.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_list_catalogs():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.ListCatalogsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_catalogs(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.ListCatalogsRequest`):
+            request (Union[google.cloud.retail_v2.types.ListCatalogsRequest, dict]):
                 The request object. Request for
                 [CatalogService.ListCatalogs][google.cloud.retail.v2.CatalogService.ListCatalogs]
                 method.
@@ -213,7 +275,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -257,18 +319,41 @@ class CatalogServiceAsyncClient:
 
     async def update_catalog(
         self,
-        request: catalog_service.UpdateCatalogRequest = None,
+        request: Union[catalog_service.UpdateCatalogRequest, dict] = None,
         *,
         catalog: gcr_catalog.Catalog = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gcr_catalog.Catalog:
         r"""Updates the [Catalog][google.cloud.retail.v2.Catalog]s.
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_update_catalog():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                catalog = retail_v2.Catalog()
+                catalog.name = "name_value"
+                catalog.display_name = "display_name_value"
+
+                request = retail_v2.UpdateCatalogRequest(
+                    catalog=catalog,
+                )
+
+                # Make the request
+                response = client.update_catalog(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.UpdateCatalogRequest`):
+            request (Union[google.cloud.retail_v2.types.UpdateCatalogRequest, dict]):
                 The request object. Request for
                 [CatalogService.UpdateCatalog][google.cloud.retail.v2.CatalogService.UpdateCatalog]
                 method.
@@ -308,7 +393,7 @@ class CatalogServiceAsyncClient:
                 The catalog configuration.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog, update_mask])
         if request is not None and has_flattened_params:
@@ -350,10 +435,10 @@ class CatalogServiceAsyncClient:
 
     async def set_default_branch(
         self,
-        request: catalog_service.SetDefaultBranchRequest = None,
+        request: Union[catalog_service.SetDefaultBranchRequest, dict] = None,
         *,
         catalog: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
@@ -398,8 +483,24 @@ class CatalogServiceAsyncClient:
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_set_default_branch():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.SetDefaultBranchRequest(
+                )
+
+                # Make the request
+                client.set_default_branch(request=request)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.SetDefaultBranchRequest`):
+            request (Union[google.cloud.retail_v2.types.SetDefaultBranchRequest, dict]):
                 The request object. Request message to set a specified
                 branch as new default_branch.
             catalog (:class:`str`):
@@ -416,7 +517,7 @@ class CatalogServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog])
         if request is not None and has_flattened_params:
@@ -453,10 +554,10 @@ class CatalogServiceAsyncClient:
 
     async def get_default_branch(
         self,
-        request: catalog_service.GetDefaultBranchRequest = None,
+        request: Union[catalog_service.GetDefaultBranchRequest, dict] = None,
         *,
         catalog: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog_service.GetDefaultBranchResponse:
@@ -469,8 +570,27 @@ class CatalogServiceAsyncClient:
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_get_default_branch():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.GetDefaultBranchRequest(
+                )
+
+                # Make the request
+                response = client.get_default_branch(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.GetDefaultBranchRequest`):
+            request (Union[google.cloud.retail_v2.types.GetDefaultBranchRequest, dict]):
                 The request object. Request message to show which branch
                 is currently the default branch.
             catalog (:class:`str`):
@@ -493,7 +613,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog])
         if request is not None and has_flattened_params:
