@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,22 +14,26 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.retail_v2.services.catalog_service import pagers
 from google.cloud.retail_v2.types import catalog
 from google.cloud.retail_v2.types import catalog as gcr_catalog
@@ -250,6 +254,73 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -300,50 +371,22 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, CatalogServiceTransport):
             # transport is a CatalogServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -355,6 +398,15 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -372,13 +424,34 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request: Union[catalog_service.ListCatalogsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListCatalogsPager:
         r"""Lists all the [Catalog][google.cloud.retail.v2.Catalog]s
         associated with the project.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_list_catalogs():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.ListCatalogsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_catalogs(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.ListCatalogsRequest, dict]):
                 The request object. Request for
@@ -413,7 +486,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -461,12 +534,35 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         *,
         catalog: gcr_catalog.Catalog = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gcr_catalog.Catalog:
         r"""Updates the [Catalog][google.cloud.retail.v2.Catalog]s.
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_update_catalog():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                catalog = retail_v2.Catalog()
+                catalog.name = "name_value"
+                catalog.display_name = "display_name_value"
+
+                request = retail_v2.UpdateCatalogRequest(
+                    catalog=catalog,
+                )
+
+                # Make the request
+                response = client.update_catalog(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.UpdateCatalogRequest, dict]):
                 The request object. Request for
@@ -508,7 +604,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
                 The catalog configuration.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog, update_mask])
         if request is not None and has_flattened_params:
@@ -553,7 +649,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request: Union[catalog_service.SetDefaultBranchRequest, dict] = None,
         *,
         catalog: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
@@ -598,6 +694,22 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_set_default_branch():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.SetDefaultBranchRequest(
+                )
+
+                # Make the request
+                client.set_default_branch(request=request)
+
         Args:
             request (Union[google.cloud.retail_v2.types.SetDefaultBranchRequest, dict]):
                 The request object. Request message to set a specified
@@ -616,7 +728,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog])
         if request is not None and has_flattened_params:
@@ -656,7 +768,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request: Union[catalog_service.GetDefaultBranchRequest, dict] = None,
         *,
         catalog: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog_service.GetDefaultBranchResponse:
@@ -669,6 +781,25 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_get_default_branch():
+                # Create a client
+                client = retail_v2.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.GetDefaultBranchRequest(
+                )
+
+                # Make the request
+                response = client.get_default_branch(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.GetDefaultBranchRequest, dict]):
                 The request object. Request message to show which branch
@@ -693,7 +824,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([catalog])
         if request is not None and has_flattened_params:
