@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.retail_v2.services.product_service import pagers
@@ -115,6 +120,42 @@ class ProductServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return ProductServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> ProductServiceTransport:
         """Returns the transport used by the client instance.
@@ -177,19 +218,43 @@ class ProductServiceAsyncClient:
 
     async def create_product(
         self,
-        request: product_service.CreateProductRequest = None,
+        request: Union[product_service.CreateProductRequest, dict] = None,
         *,
         parent: str = None,
         product: gcr_product.Product = None,
         product_id: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gcr_product.Product:
         r"""Creates a [Product][google.cloud.retail.v2.Product].
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_create_product():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                product = retail_v2.Product()
+                product.title = "title_value"
+
+                request = retail_v2.CreateProductRequest(
+                    parent="parent_value",
+                    product=product,
+                    product_id="product_id_value",
+                )
+
+                # Make the request
+                response = client.create_product(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.CreateProductRequest`):
+            request (Union[google.cloud.retail_v2.types.CreateProductRequest, dict]):
                 The request object. Request message for
                 [CreateProduct][] method.
             parent (:class:`str`):
@@ -243,7 +308,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, product, product_id])
         if request is not None and has_flattened_params:
@@ -285,17 +350,36 @@ class ProductServiceAsyncClient:
 
     async def get_product(
         self,
-        request: product_service.GetProductRequest = None,
+        request: Union[product_service.GetProductRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> product.Product:
         r"""Gets a [Product][google.cloud.retail.v2.Product].
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_get_product():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.GetProductRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_product(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.GetProductRequest`):
+            request (Union[google.cloud.retail_v2.types.GetProductRequest, dict]):
                 The request object. Request message for [GetProduct][]
                 method.
             name (:class:`str`):
@@ -329,7 +413,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -367,17 +451,37 @@ class ProductServiceAsyncClient:
 
     async def list_products(
         self,
-        request: product_service.ListProductsRequest = None,
+        request: Union[product_service.ListProductsRequest, dict] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListProductsAsyncPager:
         r"""Gets a list of [Product][google.cloud.retail.v2.Product]s.
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_list_products():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.ListProductsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_products(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.ListProductsRequest`):
+            request (Union[google.cloud.retail_v2.types.ListProductsRequest, dict]):
                 The request object. Request message for
                 [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts]
                 method.
@@ -412,7 +516,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -456,18 +560,40 @@ class ProductServiceAsyncClient:
 
     async def update_product(
         self,
-        request: product_service.UpdateProductRequest = None,
+        request: Union[product_service.UpdateProductRequest, dict] = None,
         *,
         product: gcr_product.Product = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gcr_product.Product:
         r"""Updates a [Product][google.cloud.retail.v2.Product].
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_update_product():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                product = retail_v2.Product()
+                product.title = "title_value"
+
+                request = retail_v2.UpdateProductRequest(
+                    product=product,
+                )
+
+                # Make the request
+                response = client.update_product(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.UpdateProductRequest`):
+            request (Union[google.cloud.retail_v2.types.UpdateProductRequest, dict]):
                 The request object. Request message for
                 [UpdateProduct][] method.
             product (:class:`google.cloud.retail_v2.types.Product`):
@@ -513,7 +639,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([product, update_mask])
         if request is not None and has_flattened_params:
@@ -555,17 +681,33 @@ class ProductServiceAsyncClient:
 
     async def delete_product(
         self,
-        request: product_service.DeleteProductRequest = None,
+        request: Union[product_service.DeleteProductRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes a [Product][google.cloud.retail.v2.Product].
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_delete_product():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.DeleteProductRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_product(request=request)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.DeleteProductRequest`):
+            request (Union[google.cloud.retail_v2.types.DeleteProductRequest, dict]):
                 The request object. Request message for
                 [DeleteProduct][] method.
             name (:class:`str`):
@@ -605,7 +747,7 @@ class ProductServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -642,9 +784,9 @@ class ProductServiceAsyncClient:
 
     async def import_products(
         self,
-        request: import_config.ImportProductsRequest = None,
+        request: Union[import_config.ImportProductsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -658,8 +800,36 @@ class ProductServiceAsyncClient:
         [Product][google.cloud.retail.v2.Product]s to be successfully
         updated.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_import_products():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                input_config = retail_v2.ProductInputConfig()
+                input_config.product_inline_source.products.title = "title_value"
+
+                request = retail_v2.ImportProductsRequest(
+                    parent="parent_value",
+                    input_config=input_config,
+                )
+
+                # Make the request
+                operation = client.import_products(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.ImportProductsRequest`):
+            request (Union[google.cloud.retail_v2.types.ImportProductsRequest, dict]):
                 The request object. Request message for Import methods.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
@@ -722,11 +892,11 @@ class ProductServiceAsyncClient:
 
     async def set_inventory(
         self,
-        request: product_service.SetInventoryRequest = None,
+        request: Union[product_service.SetInventoryRequest, dict] = None,
         *,
         inventory: product.Product = None,
         set_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -778,8 +948,35 @@ class ProductServiceAsyncClient:
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_set_inventory():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                inventory = retail_v2.Product()
+                inventory.title = "title_value"
+
+                request = retail_v2.SetInventoryRequest(
+                    inventory=inventory,
+                )
+
+                # Make the request
+                operation = client.set_inventory(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.SetInventoryRequest`):
+            request (Union[google.cloud.retail_v2.types.SetInventoryRequest, dict]):
                 The request object. Request message for [SetInventory][]
                 method.
             inventory (:class:`google.cloud.retail_v2.types.Product`):
@@ -857,7 +1054,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([inventory, set_mask])
         if request is not None and has_flattened_params:
@@ -907,10 +1104,10 @@ class ProductServiceAsyncClient:
 
     async def add_fulfillment_places(
         self,
-        request: product_service.AddFulfillmentPlacesRequest = None,
+        request: Union[product_service.AddFulfillmentPlacesRequest, dict] = None,
         *,
         product: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -933,8 +1130,34 @@ class ProductServiceAsyncClient:
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_add_fulfillment_places():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.AddFulfillmentPlacesRequest(
+                    product="product_value",
+                    type_="type__value",
+                    place_ids=['place_ids_value_1', 'place_ids_value_2'],
+                )
+
+                # Make the request
+                operation = client.add_fulfillment_places(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.AddFulfillmentPlacesRequest`):
+            request (Union[google.cloud.retail_v2.types.AddFulfillmentPlacesRequest, dict]):
                 The request object. Request message for
                 [AddFulfillmentPlaces][] method.
             product (:class:`str`):
@@ -960,13 +1183,13 @@ class ProductServiceAsyncClient:
             google.api_core.operation_async.AsyncOperation:
                 An object representing a long-running operation.
 
-                The result type for the operation will be :class:`google.cloud.retail_v2.types.AddFulfillmentPlacesResponse` Response of the RemoveFulfillmentPlacesRequest. Currently empty because
+                The result type for the operation will be :class:`google.cloud.retail_v2.types.AddFulfillmentPlacesResponse` Response of the AddFulfillmentPlacesRequest. Currently empty because
                    there is no meaningful response populated from the
                    [AddFulfillmentPlaces][] method.
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([product])
         if request is not None and has_flattened_params:
@@ -1012,10 +1235,10 @@ class ProductServiceAsyncClient:
 
     async def remove_fulfillment_places(
         self,
-        request: product_service.RemoveFulfillmentPlacesRequest = None,
+        request: Union[product_service.RemoveFulfillmentPlacesRequest, dict] = None,
         *,
         product: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -1038,8 +1261,34 @@ class ProductServiceAsyncClient:
         `here <https://cloud.google.com/contact>`__ to contact cloud
         sales if you are interested in using Retail Search.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_remove_fulfillment_places():
+                # Create a client
+                client = retail_v2.ProductServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.RemoveFulfillmentPlacesRequest(
+                    product="product_value",
+                    type_="type__value",
+                    place_ids=['place_ids_value_1', 'place_ids_value_2'],
+                )
+
+                # Make the request
+                operation = client.remove_fulfillment_places(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.RemoveFulfillmentPlacesRequest`):
+            request (Union[google.cloud.retail_v2.types.RemoveFulfillmentPlacesRequest, dict]):
                 The request object. Request message for
                 [RemoveFulfillmentPlaces][] method.
             product (:class:`str`):
@@ -1071,7 +1320,7 @@ class ProductServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([product])
         if request is not None and has_flattened_params:
