@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api import httpbody_pb2  # type: ignore
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -113,6 +118,42 @@ class UserEventServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return UserEventServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> UserEventServiceTransport:
         """Returns the transport used by the client instance.
@@ -175,16 +216,40 @@ class UserEventServiceAsyncClient:
 
     async def write_user_event(
         self,
-        request: user_event_service.WriteUserEventRequest = None,
+        request: Union[user_event_service.WriteUserEventRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> user_event.UserEvent:
         r"""Writes a single user event.
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_write_user_event():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                user_event = retail_v2.UserEvent()
+                user_event.event_type = "event_type_value"
+                user_event.visitor_id = "visitor_id_value"
+
+                request = retail_v2.WriteUserEventRequest(
+                    parent="parent_value",
+                    user_event=user_event,
+                )
+
+                # Make the request
+                response = client.write_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.WriteUserEventRequest`):
+            request (Union[google.cloud.retail_v2.types.WriteUserEventRequest, dict]):
                 The request object. Request message for WriteUserEvent
                 method.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -226,9 +291,9 @@ class UserEventServiceAsyncClient:
 
     async def collect_user_event(
         self,
-        request: user_event_service.CollectUserEventRequest = None,
+        request: Union[user_event_service.CollectUserEventRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> httpbody_pb2.HttpBody:
@@ -239,8 +304,29 @@ class UserEventServiceAsyncClient:
         pixel and Google Tag Manager. Users should not call this
         method directly.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_collect_user_event():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.CollectUserEventRequest(
+                    parent="parent_value",
+                    user_event="user_event_value",
+                )
+
+                # Make the request
+                response = client.collect_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.CollectUserEventRequest`):
+            request (Union[google.cloud.retail_v2.types.CollectUserEventRequest, dict]):
                 The request object. Request message for CollectUserEvent
                 method.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -275,10 +361,11 @@ class UserEventServiceAsyncClient:
                       }
 
                       service ResourceService {
-                         rpc GetResource(GetResourceRequest) returns
-                         (google.api.HttpBody); rpc
-                         UpdateResource(google.api.HttpBody) returns
-                         (google.protobuf.Empty);
+                         rpc GetResource(GetResourceRequest)
+                            returns (google.api.HttpBody);
+
+                         rpc UpdateResource(google.api.HttpBody)
+                            returns (google.protobuf.Empty);
 
                       }
 
@@ -323,9 +410,9 @@ class UserEventServiceAsyncClient:
 
     async def purge_user_events(
         self,
-        request: purge_config.PurgeUserEventsRequest = None,
+        request: Union[purge_config.PurgeUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -335,8 +422,33 @@ class UserEventServiceAsyncClient:
         or days to complete. To test a filter, use the list
         command first.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_purge_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.PurgeUserEventsRequest(
+                    parent="parent_value",
+                    filter="filter_value",
+                )
+
+                # Make the request
+                operation = client.purge_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.PurgeUserEventsRequest`):
+            request (Union[google.cloud.retail_v2.types.PurgeUserEventsRequest, dict]):
                 The request object. Request message for PurgeUserEvents
                 method.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -397,9 +509,9 @@ class UserEventServiceAsyncClient:
 
     async def import_user_events(
         self,
-        request: import_config.ImportUserEventsRequest = None,
+        request: Union[import_config.ImportUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -411,8 +523,37 @@ class UserEventServiceAsyncClient:
         successfully inserted. Operation.metadata is of type
         ImportMetadata.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_import_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                input_config = retail_v2.UserEventInputConfig()
+                input_config.user_event_inline_source.user_events.event_type = "event_type_value"
+                input_config.user_event_inline_source.user_events.visitor_id = "visitor_id_value"
+
+                request = retail_v2.ImportUserEventsRequest(
+                    parent="parent_value",
+                    input_config=input_config,
+                )
+
+                # Make the request
+                operation = client.import_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.ImportUserEventsRequest`):
+            request (Union[google.cloud.retail_v2.types.ImportUserEventsRequest, dict]):
                 The request object. Request message for the
                 ImportUserEvents request.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -447,9 +588,9 @@ class UserEventServiceAsyncClient:
                     core_exceptions.DeadlineExceeded,
                     core_exceptions.ServiceUnavailable,
                 ),
-                deadline=300.0,
+                deadline=600.0,
             ),
-            default_timeout=300.0,
+            default_timeout=600.0,
             client_info=DEFAULT_CLIENT_INFO,
         )
 
@@ -475,9 +616,9 @@ class UserEventServiceAsyncClient:
 
     async def rejoin_user_events(
         self,
-        request: user_event_service.RejoinUserEventsRequest = None,
+        request: Union[user_event_service.RejoinUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -492,8 +633,32 @@ class UserEventServiceAsyncClient:
         used to correct events joined with wrong product
         catalog.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_rejoin_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.RejoinUserEventsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.rejoin_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.retail_v2.types.RejoinUserEventsRequest`):
+            request (Union[google.cloud.retail_v2.types.RejoinUserEventsRequest, dict]):
                 The request object. Request message for RejoinUserEvents
                 method.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
