@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,22 +14,26 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api import httpbody_pb2  # type: ignore
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -262,6 +266,73 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -312,50 +383,22 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, UserEventServiceTransport):
             # transport is a UserEventServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -367,6 +410,15 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -383,12 +435,36 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         request: Union[user_event_service.WriteUserEventRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> user_event.UserEvent:
         r"""Writes a single user event.
 
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_write_user_event():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                user_event = retail_v2.UserEvent()
+                user_event.event_type = "event_type_value"
+                user_event.visitor_id = "visitor_id_value"
+
+                request = retail_v2.WriteUserEventRequest(
+                    parent="parent_value",
+                    user_event=user_event,
+                )
+
+                # Make the request
+                response = client.write_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.WriteUserEventRequest, dict]):
                 The request object. Request message for WriteUserEvent
@@ -435,7 +511,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         request: Union[user_event_service.CollectUserEventRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> httpbody_pb2.HttpBody:
@@ -446,6 +522,27 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         pixel and Google Tag Manager. Users should not call this
         method directly.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_collect_user_event():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.CollectUserEventRequest(
+                    parent="parent_value",
+                    user_event="user_event_value",
+                )
+
+                # Make the request
+                response = client.collect_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.CollectUserEventRequest, dict]):
                 The request object. Request message for CollectUserEvent
@@ -482,10 +579,11 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
                       }
 
                       service ResourceService {
-                         rpc GetResource(GetResourceRequest) returns
-                         (google.api.HttpBody); rpc
-                         UpdateResource(google.api.HttpBody) returns
-                         (google.protobuf.Empty);
+                         rpc GetResource(GetResourceRequest)
+                            returns (google.api.HttpBody);
+
+                         rpc UpdateResource(google.api.HttpBody)
+                            returns (google.protobuf.Empty);
 
                       }
 
@@ -533,7 +631,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         request: Union[purge_config.PurgeUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
@@ -543,6 +641,31 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         or days to complete. To test a filter, use the list
         command first.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_purge_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.PurgeUserEventsRequest(
+                    parent="parent_value",
+                    filter="filter_value",
+                )
+
+                # Make the request
+                operation = client.purge_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.PurgeUserEventsRequest, dict]):
                 The request object. Request message for PurgeUserEvents
@@ -598,7 +721,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         request: Union[import_config.ImportUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
@@ -610,6 +733,35 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         successfully inserted. Operation.metadata is of type
         ImportMetadata.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_import_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                input_config = retail_v2.UserEventInputConfig()
+                input_config.user_event_inline_source.user_events.event_type = "event_type_value"
+                input_config.user_event_inline_source.user_events.visitor_id = "visitor_id_value"
+
+                request = retail_v2.ImportUserEventsRequest(
+                    parent="parent_value",
+                    input_config=input_config,
+                )
+
+                # Make the request
+                operation = client.import_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.ImportUserEventsRequest, dict]):
                 The request object. Request message for the
@@ -667,7 +819,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         request: Union[user_event_service.RejoinUserEventsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
@@ -682,6 +834,30 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         used to correct events joined with wrong product
         catalog.
 
+
+        .. code-block:: python
+
+            from google.cloud import retail_v2
+
+            def sample_rejoin_user_events():
+                # Create a client
+                client = retail_v2.UserEventServiceClient()
+
+                # Initialize request argument(s)
+                request = retail_v2.RejoinUserEventsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.rejoin_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.retail_v2.types.RejoinUserEventsRequest, dict]):
                 The request object. Request message for RejoinUserEvents
