@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -102,7 +102,7 @@ class ColorInfo(proto.Message):
             only 1 color. May consider using single "Mixed" instead of
             multiple values.
 
-            A maximum of 5 colors are allowed. Each value must be a
+            A maximum of 25 colors are allowed. Each value must be a
             UTF-8 encoded string with a length limit of 128 characters.
             Otherwise, an INVALID_ARGUMENT error is returned.
 
@@ -158,6 +158,8 @@ class CustomAttribute(proto.Message):
             Only set if type
             [text][google.cloud.retail.v2.CustomAttribute.text] is set.
             Otherwise, a INVALID_ARGUMENT error is returned.
+
+            This field is a member of `oneof`_ ``_searchable``.
         indexable (bool):
             If true, custom attribute values are indexed, so that it can
             be filtered, faceted or boosted in
@@ -172,6 +174,8 @@ class CustomAttribute(proto.Message):
             and
             [SearchRequest.boost_spec][google.cloud.retail.v2.SearchRequest.boost_spec]
             for more details.
+
+            This field is a member of `oneof`_ ``_indexable``.
     """
 
     text = proto.RepeatedField(proto.STRING, number=1,)
@@ -213,9 +217,9 @@ class FulfillmentInfo(proto.Message):
             or the region IDs for
             [FulfillmentInfo.type.same-day-delivery][google.cloud.retail.v2.FulfillmentInfo.type].
 
-            A maximum of 2000 values are allowed. Each value must be a
-            string with a length limit of 10 characters, matching the
-            pattern [a-zA-Z0-9\_-]+, such as "store1" or "REGION-2".
+            A maximum of 3000 values are allowed. Each value must be a
+            string with a length limit of 30 characters, matching the
+            pattern ``[a-zA-Z0-9_-]+``, such as "store1" or "REGION-2".
             Otherwise, an INVALID_ARGUMENT error is returned.
     """
 
@@ -258,15 +262,30 @@ class Image(proto.Message):
 class Interval(proto.Message):
     r"""A floating point interval.
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         minimum (float):
             Inclusive lower bound.
+
+            This field is a member of `oneof`_ ``min``.
         exclusive_minimum (float):
             Exclusive lower bound.
+
+            This field is a member of `oneof`_ ``min``.
         maximum (float):
             Inclusive upper bound.
+
+            This field is a member of `oneof`_ ``max``.
         exclusive_maximum (float):
             Exclusive upper bound.
+
+            This field is a member of `oneof`_ ``max``.
     """
 
     minimum = proto.Field(proto.DOUBLE, number=1, oneof="min",)
@@ -509,7 +528,7 @@ class Promotion(proto.Message):
 
             The value value must be a UTF-8 encoded string with a length
             limit of 128 characters, and match the pattern:
-            [a-zA-Z][a-zA-Z0-9\_]*. For example, id0LikeThis or
+            ``[a-zA-Z][a-zA-Z0-9_]*``. For example, id0LikeThis or
             ID_1_LIKE_THIS. Otherwise, an INVALID_ARGUMENT error is
             returned.
 
