@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,6 +29,13 @@ class Product(proto.Message):
     r"""Product captures all metadata information of items to be
     recommended or searched.
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
+
+    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
     Attributes:
         expire_time (google.protobuf.timestamp_pb2.Timestamp):
             The timestamp when this product becomes unavailable for
@@ -44,12 +51,21 @@ class Product(proto.Message):
             and
             [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
 
+            [expire_time][google.cloud.retail.v2.Product.expire_time]
+            must be later than
+            [available_time][google.cloud.retail.v2.Product.available_time]
+            and
+            [publish_time][google.cloud.retail.v2.Product.publish_time],
+            otherwise an INVALID_ARGUMENT error is thrown.
+
             Google Merchant Center property
             `expiration_date <https://support.google.com/merchants/answer/6324499>`__.
+
+            This field is a member of `oneof`_ ``expiration``.
         ttl (google.protobuf.duration_pb2.Duration):
             Input only. The TTL (time to live) of the product.
 
-            If it is set,
+            If it is set, it must be a non-negative value, and
             [expire_time][google.cloud.retail.v2.Product.expire_time] is
             set as current timestamp plus
             [ttl][google.cloud.retail.v2.Product.ttl]. The derived
@@ -66,11 +82,11 @@ class Product(proto.Message):
             [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
             and
             [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+
+            This field is a member of `oneof`_ ``expiration``.
         name (str):
             Immutable. Full resource name of the product, such as
             ``projects/*/locations/global/catalogs/default_catalog/branches/default_branch/products/product_id``.
-
-            The branch ID must be "default_branch".
         id (str):
             Immutable. [Product][google.cloud.retail.v2.Product]
             identifier, which is the final component of
@@ -256,8 +272,8 @@ class Product(proto.Message):
             -  The key must be a UTF-8 encoded string with a length
                limit of 128 characters.
             -  For indexable attribute, the key must match the pattern:
-               [a-zA-Z0-9][a-zA-Z0-9\_]*. For example, key0LikeThis or
-               KEY_1_LIKE_THIS.
+               ``[a-zA-Z0-9][a-zA-Z0-9_]*``. For example, key0LikeThis
+               or KEY_1_LIKE_THIS.
         tags (Sequence[str]):
             Custom tags associated with the product.
 
