@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -357,7 +357,7 @@ class SetInventoryRequest(proto.Message):
             found, the inventory update will still be processed and
             retained for at most 1 day until the
             [Product][google.cloud.retail.v2.Product] is created. If set
-            to false, an INVALID_ARGUMENT error is returned if the
+            to false, a NOT_FOUND error is returned if the
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
@@ -428,9 +428,9 @@ class AddFulfillmentPlacesRequest(proto.Message):
 
             At least 1 value is required, and a maximum of 2000 values
             are allowed. Each value must be a string with a length limit
-            of 10 characters, matching the pattern [a-zA-Z0-9\_-]+, such
-            as "store1" or "REGION-2". Otherwise, an INVALID_ARGUMENT
-            error is returned.
+            of 10 characters, matching the pattern ``[a-zA-Z0-9_-]+``,
+            such as "store1" or "REGION-2". Otherwise, an
+            INVALID_ARGUMENT error is returned.
 
             If the total number of place IDs exceeds 2000 for this
             [type][google.cloud.retail.v2.AddFulfillmentPlacesRequest.type]
@@ -446,7 +446,7 @@ class AddFulfillmentPlacesRequest(proto.Message):
             fulfillment information will still be processed and retained
             for at most 1 day and processed once the
             [Product][google.cloud.retail.v2.Product] is created. If set
-            to false, an INVALID_ARGUMENT error is returned if the
+            to false, a NOT_FOUND error is returned if the
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
@@ -466,8 +466,8 @@ class AddFulfillmentPlacesMetadata(proto.Message):
 
 
 class AddFulfillmentPlacesResponse(proto.Message):
-    r"""Response of the RemoveFulfillmentPlacesRequest. Currently empty
-    because there is no meaningful response populated from the
+    r"""Response of the AddFulfillmentPlacesRequest. Currently empty because
+    there is no meaningful response populated from the
     [AddFulfillmentPlaces][] method.
 
     """
@@ -517,9 +517,9 @@ class RemoveFulfillmentPlacesRequest(proto.Message):
 
             At least 1 value is required, and a maximum of 2000 values
             are allowed. Each value must be a string with a length limit
-            of 10 characters, matching the pattern [a-zA-Z0-9\_-]+, such
-            as "store1" or "REGION-2". Otherwise, an INVALID_ARGUMENT
-            error is returned.
+            of 10 characters, matching the pattern ``[a-zA-Z0-9_-]+``,
+            such as "store1" or "REGION-2". Otherwise, an
+            INVALID_ARGUMENT error is returned.
         remove_time (google.protobuf.timestamp_pb2.Timestamp):
             The time when the fulfillment updates are
             issued, used to prevent out-of-order updates on
@@ -531,7 +531,7 @@ class RemoveFulfillmentPlacesRequest(proto.Message):
             fulfillment information will still be processed and retained
             for at most 1 day and processed once the
             [Product][google.cloud.retail.v2.Product] is created. If set
-            to false, an INVALID_ARGUMENT error is returned if the
+            to false, a NOT_FOUND error is returned if the
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
