@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,12 +36,9 @@ class SearchRequest(proto.Message):
             Required. The resource name of the search engine placement,
             such as
             ``projects/*/locations/global/catalogs/default_catalog/placements/default_search``.
-            This field is used to identify the set of models that will
-            be used to make the search.
-
-            We currently support one placement with the following ID:
-
-            -  ``default_search``.
+            This field is used to identify the serving configuration
+            name and the set of models that will be used to make the
+            search.
         branch (str):
             The branch resource name, such as
             ``projects/*/locations/global/catalogs/default_catalog/branches/0``.
@@ -94,7 +91,8 @@ class SearchRequest(proto.Message):
             The filter syntax consists of an expression language for
             constructing a predicate from one or more fields of the
             products being filtered. Filter expression is
-            case-sensitive.
+            case-sensitive. See more details at this `user
+            guide <https://cloud.google.com/retail/docs/filter-and-order#filter>`__.
 
             If this field is unrecognizable, an INVALID_ARGUMENT is
             returned.
@@ -116,7 +114,8 @@ class SearchRequest(proto.Message):
             ordered by a field in an
             [Product][google.cloud.retail.v2.Product] object. Leave it
             unset if ordered by relevance. OrderBy expression is
-            case-sensitive.
+            case-sensitive. See more details at this `user
+            guide <https://cloud.google.com/retail/docs/filter-and-order#order>`__.
 
             If this field is unrecognizable, an INVALID_ARGUMENT is
             returned.
@@ -134,12 +133,21 @@ class SearchRequest(proto.Message):
             Contact Retail Search support team if you are
             interested in using dynamic facet feature.
         boost_spec (google.cloud.retail_v2.types.SearchRequest.BoostSpec):
-            Boost specification to boost certain
-            products.
+            Boost specification to boost certain products. See more
+            details at this `user
+            guide <https://cloud.google.com/retail/docs/boosting>`__.
+
+            Notice that if both [ServingConfig.boost_control_ids][] and
+            [SearchRequest.boost_spec] are set, the boost conditions
+            from both places are evaluated. If a search request matches
+            multiple boost conditions, the final boost score is equal to
+            the sum of the boost scores from all matched boost
+            conditions.
         query_expansion_spec (google.cloud.retail_v2.types.SearchRequest.QueryExpansionSpec):
-            The query expansion specification that
-            specifies the conditions under which query
-            expansion will occur.
+            The query expansion specification that specifies the
+            conditions under which query expansion will occur. See more
+            details at this `user
+            guide <https://cloud.google.com/retail/docs/result-size#query_expansion>`__.
         variant_rollup_keys (Sequence[str]):
             The keys to fetch and rollup the matching
             [variant][google.cloud.retail.v2.Product.Type.VARIANT]
@@ -165,6 +173,10 @@ class SearchRequest(proto.Message):
             -  price
             -  originalPrice
             -  discount
+            -  variantId
+            -  inventory(place_id,price)
+            -  inventory(place_id,attributes.key), where key is any key
+               in the [Product.inventories.attributes][] map.
             -  attributes.key, where key is any key in the
                [Product.attributes][google.cloud.retail.v2.Product.attributes]
                map.
@@ -230,8 +242,18 @@ class SearchRequest(proto.Message):
             promotions. For instance, a special sale page may have the
             category hierarchy: "pageCategories" : ["Sales > 2017 Black
             Friday Deals"].
+        search_mode (google.cloud.retail_v2.types.SearchRequest.SearchMode):
+            The search mode of the search request. If not
+            specified, a single search request triggers both
+            product search and faceted search.
     """
 
+    class SearchMode(proto.Enum):
+        r"""The search mode of each search request."""
+        SEARCH_MODE_UNSPECIFIED = 0
+        PRODUCT_SEARCH_ONLY = 1
+        FACETED_SEARCH_ONLY = 2
+
     class FacetSpec(proto.Message):
         r"""A facet specification to perform faceted search.
 
@@ -338,6 +360,7 @@ class SearchRequest(proto.Message):
                        -  "customFulfillment3"
                        -  "customFulfillment4"
                        -  "customFulfillment5"
+                       -  "inventory(place_id,attributes.key)"
 
                     -  numerical_field =
 
@@ -345,7 +368,9 @@ class SearchRequest(proto.Message):
                        -  "discount"
                        -  "rating"
                        -  "ratingCount"
-                       -  "attributes.key".
+                       -  "attributes.key"
+                       -  "inventory(place_id,price)"
+                       -  "inventory(place_id,attributes.key)".
                 intervals (Sequence[google.cloud.retail_v2.types.Interval]):
                     Set only if values should be bucketized into
                     intervals. Must be set for facets with numerical
@@ -586,6 +611,7 @@ class SearchRequest(proto.Message):
     )
     variant_rollup_keys = proto.RepeatedField(proto.STRING, number=17,)
     page_categories = proto.RepeatedField(proto.STRING, number=23,)
+    search_mode = proto.Field(proto.ENUM, number=31, enum=SearchMode,)
 
 
 class SearchResponse(proto.Message):
@@ -738,13 +764,24 @@ class SearchResponse(proto.Message):
         class FacetValue(proto.Message):
             r"""A facet value which contains value names and their count.
 
+            This message has `oneof`_ fields (mutually exclusive fields).
+            For each oneof, at most one member field can be set at the same time.
+            Setting any member of the oneof automatically clears all other
+            members.
+
+            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
+
             Attributes:
                 value (str):
                     Text value of a facet, such as "Black" for
                     facet "colorFamilies".
+
+                    This field is a member of `oneof`_ ``facet_value``.
                 interval (google.cloud.retail_v2.types.Interval):
                     Interval value for a facet, such as [10, 20) for facet
                     "price".
+
+                    This field is a member of `oneof`_ ``facet_value``.
                 count (int):
                     Number of items that have this facet value.
             """
@@ -772,7 +809,7 @@ class SearchResponse(proto.Message):
             pinned_result_count (int):
                 Number of pinned results. This field will only be set when
                 expansion happens and
-                [SearchRequest.query_expansion_spec.pin_unexpanded_results][]
+                [SearchRequest.QueryExpansionSpec.pin_unexpanded_results][google.cloud.retail.v2.SearchRequest.QueryExpansionSpec.pin_unexpanded_results]
                 is set to true.
         """
 
